<?php

App::uses('ModelValidator', 'Model');
App::uses('Session', 'Model/Datasource');
App::uses('AppBehavior', 'Model/Behavior');

class CustomFieldsDataBehavior extends AppBehavior {

/**
 * save the error state incase it somehow
 * gets through and goes to afterSave
 */
	public $validationErrors = array();

	public $getFieldData = false;

	public $fieldDataSyntaxFormat = false;

/**
 * Field types that support multiple values.
 *
 * @var array
 */
	protected $_multipleDataFieldTypes = [
		'chosen_multiselect',
		'multicheck',
		'multiselect',
	];

/**
 * Setup this behavior with the specified configuration settings.
 *
 * @param Model $model Model using this behavior
 * @param array $config Configuration settings for $model
 * @return void
 */
	public function setup(Model $Model, $config = array()) {
		$this->_linkModels($Model);
	}

/**
 * validate the custom fields and die out if needed
 *
 * beforeSave is called before a model is saved. Returning false from a beforeSave callback
 * will abort the save operation.
 *
 * @param Model $model Model using this behavior
 * @param array $options Options passed from Model::save().
 * @return mixed False if the operation should abort. Any other result will continue.
 * @see Model::save()
 */
	public function beforeSave(Model $Model, $options = array()) {
		$return = parent::beforeSave($Model, $options);

		if (! empty($Model->data[$Model->alias]['CustomFieldData'])) {
			$CustomFieldData = EvClassRegistry::init('EvCustomFields.CustomFieldData');
			$FieldData = EvClassRegistry::init('EvCustomFields.FieldData');
			$Field = EvClassRegistry::init('EvCustomFields.Field');

			$validationRules = $Field->getValidation(array_keys($Model->data[$Model->alias]['CustomFieldData']));

			foreach ($Model->data[$Model->alias]['CustomFieldData'] as $fieldId => $field) {
				$compiledRules = $this->compileCFValidationRules($validationRules, $fieldId);

				$validates = $this->validateCustomField(
					$FieldData,
					$Model->data[$Model->alias]['CustomFieldData'][$fieldId]['FieldData']['value'],
					$compiledRules
				);

				if (! $validates) {
					$error = true;
					$CustomFieldData->validationErrors[$fieldId]['FieldData']['value'] = $FieldData->validationErrors['value'];
				}
			}
		}

		return (
			$return &&
			(! isset($error) || $error === false)
		);
	}

/**
 * Save the custom field data
 *
 * @param Model $model Model using this behavior
 * @param bool $created True if this save created a new record
 * @param array $options Options passed from Model::save().
 * @return bool
 * @see Model::save()
 */
	public function afterSave(Model $Model, $created, $options = array()) {
		$return = parent::afterSave($Model, $created, $options);
		$result = true;

		if (! empty($Model->data[$Model->alias]['CustomFieldData'])) {
			$FieldData = EvClassRegistry::init('EvCustomFields.FieldData');
			$fieldTypes = EvClassRegistry::init('EvCustomFields.Field')->find('list', [
				'fields' => ['id', 'type']
			]);

			// setup the model / model id
			$modelInfo = array(
				'model' => EvClassRegistry::getNameFromModel($Model),
				'model_id' => $Model->data[$Model->alias][$Model->primaryKey]
			);

			foreach ($Model->data[$Model->alias]['CustomFieldData'] as $fieldId => &$field) {
				$Model->data[$Model->alias]['CustomFieldData'][$fieldId]['FieldData']['field_id'] = $fieldId;
				$Model->data[$Model->alias]['CustomFieldData'][$fieldId]['FieldData']['model'] = $modelInfo['model'];
				$Model->data[$Model->alias]['CustomFieldData'][$fieldId]['FieldData']['model_id'] = $modelInfo['model_id'];
				$field['FieldData'] = $this->_prepareFieldForSave($field['FieldData'], $fieldTypes);
			}

			$result = $FieldData->saveMany(
				$Model->data[$Model->alias]['CustomFieldData']
			);
		}

		return ($return && $result);
	}

/**
 * Prepares custom field data for save by JSON encoding data for multiple value fields. This method
 * is called in the afterSave callback.
 *
 * @param array $customField Custom field data
 * @param array $fieldTypes Field types
 * @return array
 */
	protected function _prepareFieldForSave($customField, $fieldTypes) {
		if (in_array($fieldTypes[$customField['field_id']], $this->_multipleDataFieldTypes)) {
			$customField['value'] = json_encode($customField['value']);
		}

		return $customField;
	}

/**
 * Prepares custom field data for display by extracting JSON encoded data for multiple value
 * fields. This method is called in the afterFind callback.
 *
 * @param array $customFieldData Custom field data
 * @return array
 */
	protected function _prepareFieldForDisplay($customFieldData) {
		if (in_array($customFieldData['Field']['type'], $this->_multipleDataFieldTypes)) {
			$customFieldData['FieldData']['value'] = json_decode($customFieldData['FieldData']['value'], true);
		}

		return $customFieldData;
	}

/**
 * validate the custom field data
 *
 * @param   Object  FieldData Model
 * @param   mixed   The actual form value
 * @param   array   Compiled array of rules to validate on
 * @return  bool
 */
	public function validateCustomField($FieldData, $value, $rules) {
		$FieldData->clear();
		$FieldData->set('value', $value);

		$Validator = new ModelValidator($FieldData);
		$Validator->add(
			'value',
			$rules
		);

		return $Validator->validates();
	}

/**
 * compile the validation rules
 *
 * @param   array   Array of all the vlaidation rules for all custom fields we are saving
 * @param   int     Field id of the current field we are saving
 * @return  array   Compiled array of validation rules to apply to field
 */
	public function compileCFValidationRules($validation, $fieldId) {
		$compiledRules = array();

		foreach ($validation[$fieldId] as $rule) {
			if (empty($rule['rule'])) {
				continue;
			}

			$ruleType = $rule['rule'];
			if (! empty($rule['values'])) {
				$values = explode(',', $rule['values']);
				$ruleType = array_merge(array($rule['rule']), $values);
			}

			$compiledRules[$rule['rule']] = array(
				'rule' => $ruleType,
				'message' => $rule['message']
			);
		}

		return $compiledRules;
	}

/**
 * beforeFind can be used to cancel find operations, or modify the query that will be executed.
 * By returning null/false you can abort a find. By returning an array you can modify/replace the query
 * that is going to be run.
 *
 * @param Model $model Model using this behavior
 * @param array $query Data used to execute this query, i.e. conditions, order, etc.
 * @return bool|array False or null will abort the operation. You can return an array to replace the
 *   $query that will be eventually run.
 */
	public function beforeFind(Model $Model, $query) {
		$this->getFieldData = false;
		$this->fieldDataSyntaxFormat = false;

		if (isset($query['customFields']) && $query['customFields'] === true) {
			$this->getFieldData = true;
		}

		if (isset($query['customFieldsSyntaxFormat']) && $query['customFieldsSyntaxFormat'] === true) {
			$this->fieldDataSyntaxFormat = true;
		}

		return $query;
	}

/**
 * define an afterFind to retrieve hte custom field data for the item
 *
 * @param Model $model Model using this behavior
 * @param mixed $results The results of the find operation
 * @param bool $primary Whether this model is being queried directly (vs. being queried as an association)
 * @return mixed An array value will replace the value of $results - any other value will be ignored.
 */
	public function afterFind(Model $Model, $results, $primary = false) {
		if ($this->getFieldData !== true) {
			return $results;
		}

		// get all the model ids
		$modelIds = Hash::extract($results, '{n}.' . $Model->alias . '.' . $Model->primaryKey);

		if (empty($modelIds)) {
			return $results;
		}

		// get all the fields and data where id IN model_id
		$FieldData = EvClassRegistry::init('EvCustomFields.FieldData');
		$customData = $FieldData->find(
			'all',
			array(
				'contain' => 'Field',
				'conditions' => array(
					'FieldData.model' => EvClassRegistry::getNameFromModel($Model),
					'FieldData.model_id' => $modelIds
				),
				'order' => 'Field.sequence ASC'
			)
		);

		if (! empty($customData)) {
			$customData = Hash::combine(
				$customData,
				'{n}.FieldData.field_id',
				'{n}',
				'{n}.FieldData.model_id'
			);

			// loop the results and assign the field data to the result
			foreach ($results as $key => $item) {
				if (empty($item[$Model->alias][$Model->primaryKey])) {
					continue;
				}

				if (! empty($customData[$item[$Model->alias][$Model->primaryKey]])) {
					foreach ($customData[$item[$Model->alias][$Model->primaryKey]] as $fieldId => $data) {
						$results[$key]['CustomFieldData'][$fieldId] = $this->_prepareFieldForDisplay($data);
					}
				}
			}

			if ($this->fieldDataSyntaxFormat === true) {
				foreach ($results as $key => $item) {
					if (! empty($item['CustomFieldData'])) {
						$results[$key]['CustomFieldData'] = Hash::combine(
							$item['CustomFieldData'],
							'{n}.Field.name',
							'{n}.FieldData.value'
						);
					}
				}
			}
		}

		return $results;
	}

/**
 * bind the inventory model to the model
 *
 */
	protected function _linkModels(&$Model) {
		$Model->bindModel(
			array(
				'hasMany' => array(
					'CustomFields' => array(
						'className' => 'EvCustomFields.FieldData',
						'foreignKey' => 'model_id',
						'conditions' => array(
							'CustomFields.model' => EvClassRegistry::getNameFromModel($Model)
						)
					)
				)
			),
			false
		);
	}

}
